@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 2.7.  */
+/* A Bison parser, made by GNU Bison 3.0.4.  */
 
 /* Bison interface for Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
-   
+
+   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,13 +26,13 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_I386__HOME_MARK_SRC_ELFUTILS_LIBCPU_I_PARSE_H_INCLUDED
-# define YY_I386__HOME_MARK_SRC_ELFUTILS_LIBCPU_I_PARSE_H_INCLUDED
-/* Enabling traces.  */
+#ifndef YY_I386_I_PARSE_H_INCLUDED
+# define YY_I386_I_PARSE_H_INCLUDED
+/* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
 #endif
@@ -40,23 +40,22 @@
 extern int i386_debug;
 #endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     kMASK = 258,
-     kPREFIX = 259,
-     kSUFFIX = 260,
-     kSYNONYM = 261,
-     kID = 262,
-     kNUMBER = 263,
-     kPERCPERC = 264,
-     kBITFIELD = 265,
-     kCHAR = 266,
-     kSPACE = 267
-   };
+  enum yytokentype
+  {
+    kMASK = 258,
+    kPREFIX = 259,
+    kSUFFIX = 260,
+    kSYNONYM = 261,
+    kID = 262,
+    kNUMBER = 263,
+    kPERCPERC = 264,
+    kBITFIELD = 265,
+    kCHAR = 266,
+    kSPACE = 267
+  };
 #endif
 /* Tokens.  */
 #define kMASK 258
@@ -70,13 +69,12 @@ extern int i386_debug;
 #define kCHAR 266
 #define kSPACE 267
 
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+
+union YYSTYPE
 {
-/* Line 2058 of yacc.c  */
-#line 217 "/home/mark/src/elfutils/libcpu/i386_parse.y"
+#line 217 "/home/mark/src/elfutils/libcpu/i386_parse.y" /* yacc.c:1909  */
 
   unsigned long int num;
   char *str;
@@ -86,29 +84,17 @@ typedef union YYSTYPE
   struct argname *name;
   struct argument *arg;
 
+#line 88 "i386_parse.h" /* yacc.c:1909  */
+};
 
-/* Line 2058 of yacc.c  */
-#line 92 "/home/mark/src/elfutils/libcpu/i386_parse.h"
-} YYSTYPE;
+typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+
 extern YYSTYPE i386_lval;
 
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int i386_parse (void *YYPARSE_PARAM);
-#else
-int i386_parse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
 int i386_parse (void);
-#else
-int i386_parse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
 
-#endif /* !YY_I386__HOME_MARK_SRC_ELFUTILS_LIBCPU_I_PARSE_H_INCLUDED  */
+#endif /* !YY_I386_I_PARSE_H_INCLUDED  */
